from datetime import datetime
from optparse import OptionParser
import importlib.util
import pathlib
import logging 

logging.basicConfig(level = logging.INFO)

def parse_options():
    parser = OptionParser()
    parser.add_option('--end_ts', type=int)
    parser.add_option('--dir', type=str)
    parser.add_option('--filename', type=str, default='monitor.py')
    parser.add_option('--methodname', type=str, default='monitor')

    options, _ = parser.parse_args()
    return options


def monitor(dir: str, end_ts: int, filename:str="monitor.py", methodname:str="monitor"):
    """
    Get all files name filename in dir
    for each file:
        execute the method methodname with the end_ts
    :param dir: the directory to scan from
    :param end_ts: the time until it we can monitor
    :param filename: default monitor.py the file name to search in dir
    :param methodname: the methodname name to search in each file. default is monitor
    :return:
    """
    logging.info(
        f"Going to monitor all {filename} files in dir {dir} end ts is {datetime.fromtimestamp(end_ts)}")
    for pyfile in pathlib.Path(dir).glob(f'**/{filename}'):
        try:
            logging.info(f"Going to monitor {pyfile}")
            spec = importlib.util.spec_from_file_location(f"{__name__}.imported_{pyfile.stem}", pyfile)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)
            monitor_method = getattr(module, methodname)
            res = monitor_method(end_ts)
            logging.info(f"monitor ends with status {res}")
        except Exception as e:
            message = f"Couldn't monitor {pyfile} exception is {e}"
            logging.error(message)
            # you can add slack or email notification

def main(options):
    """
    Get all files name options.filename in options.dir
    for each file:
        execute the method options.methodname with the end_ts
    :param options:
    :return:
    """
    logging.info(f"Going to monitor all {options.filename} files in dir {options.dir} end ts is {datetime.fromtimestamp(options.end_ts)}")
    monitor(options.dir, options.end_ts, options.filename, options.methodname)


if __name__ == '__main__':
    options = parse_options()
    main(options)



from abc import ABC, abstractmethod
from datetime import datetime
import logging

logger = logging.getLogger()

@abstractmethod
def monitor():
    raise NotImplementedException()

class PyMonitoring:
    """
    Class for auxiliary method for monitor
    """
    @staticmethod
    def send_slack_monitoring_massage(message, channel="monitor", attachments=""):
        """
        Send message to the slack channel
        :param message: the message to send
        :param channel: the slack channel
        :param attachments:
        :return:
        """
        logger.info("implement with you slack")

    @staticmethod
    def check_table_in_sf(start_ts: int, end_ts:int, table:str, sf_client,
                               send_slack_if_no_data: bool = True, time_field_name: str= "start_ts") -> bool:
        """
        Check if there is data in the specified table between the start ts and end ts in the given time_field_name
        if there is data return True else can send slack notification and return False
        :param start_ts: the timestamp to start from
        :param end_ts: the end timestamp
        :param table: the snowflake table to look into
        :param sf_client: the snoflake client to use
        :param send_slack_if_no_data: default is True. send slack message if there is no data
        :param time_field_name: the field to check start_ts and end_ts in
        :return: True if there is Data False otherwise
        """
        query = f"select * from {table} where {time_field_name} between {start_ts} and {end_ts}"
        result = PyMonitoring.check_query_in_sf(query, sf_client, False)
        if not result and send_slack_if_no_data:
            message = f"No result for table {table} between {datetime.fromtimestamp(start_ts)} and " \
                        f"{datetime.fromtimestamp(end_ts)}" \
                        f"\nQuery is {query}" \
                        f"\nResult is {result}"
            PyMonitoring.send_slack_monitoring_massage(message)
            return False
        return True

    @staticmethod
    def check_query_in_sf(query:str, sf_client,
                          send_slack_if_no_data: bool=True)->list:
        """
        Send query to snowflake and return the query result.
        send slack message if there is no data
        :param query: the query to send the snowflake client
        :param sf_client: the snowflake client
        :param send_slack_if_no_data: default is True. send slack message if there is no data
        :return: The query result
        """
        logger.info(f"Going to run query:\n{query}")
        result = []
        #should be:
        #result = sf_client.query(query)

        if not result:
            if send_slack_if_no_data:
                message = f"No result for query {query}" \
                          f"\nResult is {result}"

                PyMonitoring.send_slack_monitoring_massage(message)
        return result




from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


setup(
    name='pymonitor-foretls',
    version='0.1.0',
    python_requires='>=3.7',
    description="pymonitoring like pytest just for monitoring",
    author="Noa gradovitch",
    author_email="noahgrad@gmail.com",
    url="https://github.com/noahgrad/pymonitor",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
    ],
    zip_safe=False
)
